addGeneIDs<-function(annotatedPeak, orgAnn, IDs2Add=c("symbol"), feature_id_type="ensembl_gene_id",silence=TRUE, mart){
	if (missing(annotatedPeak))
	{
		stop("Missing required argument annotatedPeak!",call.=FALSE)	
	}
	if(missing(orgAnn) & missing(mart)){
		stop('no annotation database selected',call.=FALSE)
	}
	if (class(annotatedPeak) == "RangedData"){
		feature_ids <- unique(annotatedPeak$feature)
	}else if (class(annotatedPeak)  ==  "character"){
		feature_ids <- unique(annotatedPeak)
	}else{
		stop("annotatedPeak needs to be RangedData type with feature variable holding the feature id or a character vector holding the IDs of the features used to annotate the peaks!",call.=FALSE)
	}
	feature_ids<-feature_ids[!is.na(feature_ids)]
	feature_ids<-feature_ids[feature_ids!=""]
	if (length(feature_ids) == 0)
	{
		stop("There is no feature column in annotatedPeak or annotatedPeak has size 0!",call.=FALSE)
		
	}
	if(!missing(orgAnn)){
		if(!grepl(".eg.db",orgAnn,ignore.case=TRUE)){
			stop('Annotation database must be *.eg.db',call.=FALSE)
		}
#	require(orgAnn,character.only = TRUE)
		orgAnn<-sub("\\.db$","",orgAnn,ignore.case=TRUE)
#get Entrez ID::entrezIDs
		if(feature_id_type=="entrez_id"){
			m_ent<-as.data.frame(feature_ids)
			m_ent<-cbind(m_ent,feature_ids)
			colnames(m_ent)<-c("entrez_id",feature_id_type)
		}else{
			prefix<-switch(feature_id_type,
						   gene_alias		= "ALIAS",
						   gene_symbol		= "SYMBOL",
						   ensembl_gene_id	= "ENSEMBL",
						   refseq_id		= "REFSEQ",
						   "UNKNOWN"
						   )
			if(prefix=="UNKNOWN"){
				stop("Currently only the following type of IDs are supported: gene_alias, 
					 ensembl_gene_id, refseq_id and gene_symbol!",call.=FALSE)
			}
			tryCatch(env<-get(paste(orgAnn,prefix,"2EG",sep="")),error = function(e){
					 stop(paste("Annotation database ",orgAnn,"2EG does not exist!\n\tPlease try to load annotation database by library(",orgAnn,".db)",sep=""),call.=FALSE)
					 })
			entrez <- AnnotationDbi::mget(feature_ids,env,ifnotfound=NA)
			gene_ids <- names(entrez)
			m_ent <- do.call(rbind,lapply(gene_ids,function(.ele){
										  r = entrez[[.ele]]
										  if(!is.na(r[1])) cbind(rep(.ele,length(r)),r)
										  else {
										  if(!silence) cat(paste("entrez id for '", .ele, "' not found\n", sep = ""))
										  c(.ele,NA)
										  }
										  }))
			m_ent<-as.data.frame(m_ent)
			m_ent<-m_ent[!is.na(m_ent[,1]),,drop=FALSE]
			colnames(m_ent)<-c(feature_id_type,"entrez_id")
		}
		entrezIDs<-as.character(m_ent$entrez_id)
		entrezIDs<-unique(entrezIDs)
		entrezIDs<-entrezIDs[!is.na(entrezIDs)]
		if(length(entrezIDs)==0){
			stop("No entrez identifier can be mapped by input data based on the feature_id_type.\nPlease consider to use correct feature_id_type, orgAnn or annotatedPeak\n",call.=FALSE);
		}
		IDs2Add<-unique(IDs2Add)
		IDs2Add<-IDs2Add[IDs2Add!=feature_id_type]
		IDs<- unique(entrezIDs[!is.na(entrezIDs)])
		for(IDtoAdd in IDs2Add){
			gc()
			x<-NULL
			cat("Adding",IDtoAdd,"... ")
			if(IDtoAdd!="entrez_id"){
				orgDB<-NULL
				tryCatch(orgDB<-get(paste(orgAnn,toupper(IDtoAdd),sep="")),error = function(e){
						 cat(paste("The IDs2Add you input, \"", IDtoAdd, "\", is not supported!\n",sep=""))
						 })
				if(is.null(orgDB)){
					IDs2Add<-IDs2Add[IDs2Add!=IDtoAdd]
					next
				}
				if(class(orgDB)!="AnnDbBimap" & class(orgDB)!="IpiAnnDbMap"){
					cat(paste("The IDs2Add you input, \"", IDtoAdd, "\", is not supported!\n",sep=""))
					IDs2Add<-IDs2Add[IDs2Add!=IDtoAdd]
					next
				}
				x <- AnnotationDbi::mget(IDs, orgDB,ifnotfound=NA)
				x <- sapply(x,paste,collapse=";")
				x <- as.data.frame(x)
				m_ent <- merge(m_ent,x,by.x="entrez_id",by.y="row.names",all.x=TRUE)
				colnames(m_ent)[length(colnames(m_ent))]<-IDtoAdd
			}
			cat("done\n")
		}
		m_ent<-m_ent[,c(feature_id_type,IDs2Add)]
	}else{
		if(missing(mart) || class(mart) !="Mart"){
			stop('No valid mart object is passed in!',call.=FALSE)
		}
		IDs2Add<-unique(IDs2Add)
		IDs2Add<-IDs2Add[IDs2Add!=feature_id_type]
		tryCatch(m_ent<-getBM(attributes=c(feature_id_type,IDs2Add),filters = feature_id_type, values = feature_ids, mart=mart),error = function(e){
				 stop(paste("Get error when calling getBM:",e,sep="\n"),call.=FALSE)
				 })
	}
	cat("prepare output ... ")
#dealing with multiple entrez_id for single feature_id
	duplicated_ids<-m_ent[duplicated(m_ent[,feature_id_type]),feature_id_type]
	if(length(duplicated_ids)>0){
		m_ent.duplicated<-m_ent[m_ent[,feature_id_type] %in% duplicated_ids,]
		m_ent.duplicated<-condenseMatrixByColnames(as.matrix(m_ent.duplicated),feature_id_type)
		m_ent<-m_ent[!(m_ent[,feature_id_type] %in% duplicated_ids),]
		m_ent<-rbind(m_ent,m_ent.duplicated)
	}
	if (class(annotatedPeak) == "RangedData"){
#rearrange m_ent by annotatedPeak$feature
#data.frame is very important for order...
		orderlist<-data.frame(annotatedPeak$feature)
		orderlist<-cbind(1:nrow(orderlist),orderlist)
		colnames(orderlist)<-c("orderid___",feature_id_type)
		m_ent<-merge(orderlist,m_ent,by=feature_id_type,all.x=T)
		m_ent<-m_ent[order(m_ent[,"orderid___"]),c(feature_id_type,IDs2Add)]
		for(IDtoAdd in IDs2Add){
			annotatedPeak$tobeAdd<-m_ent[,IDtoAdd]
			cname<-colnames(annotatedPeak)
			cname[length(cname)]<-IDtoAdd
			colnames(annotatedPeak)<-cname
		}
	}else{
		annotatedPeak<-m_ent
	}
	cat("done\n")
	annotatedPeak
}